@@ -24,7 +24,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: logging.c,v 1.9 2005/05/15 08:00:49 steveu Exp $
+ * $Id: logging.c,v 1.10 2005/05/22 05:45:32 steveu Exp $
  */
 
 /*! \file uni-call.h */
@@ -101,12 +101,10 @@ int uc_log(uc_t *uc, int level, const char *format, ...)
         /*endif*/
         if ((uc->logging_level & UC_LOG_SHOW_VARIANT))
         {
-#if 0
             strcpy(msg + len, uc_protocol_variant2str(uc->protocol_class, uc->protocol_variant));
             len += strlen(msg + len);
             strcpy(msg + len, " ");
             len += strlen(msg + len);
-#endif
         }
         /*endif*/
         if ((uc->logging_level & UC_LOG_SHOW_TAG)  &&  uc->logging_tag)
@@ -495,6 +493,25 @@ const char *uc_location2str(int code)
 }
 /*- End of function --------------------------------------------------------*/
 
+const char *uc_calling_party_category2str(int code)
+{
+    static msgtype_t codes[] =
+    {
+        { UC_CALLER_CATEGORY_NATIONAL_SUBSCRIBER_CALL, "National subscriber call" },
+        { UC_CALLER_CATEGORY_NATIONAL_PRIORITY_SUBSCRIBER_CALL, "National priority subscriber call" },
+        { UC_CALLER_CATEGORY_NATIONAL_MAINTENANCE_CALL, "National maintenance call" },
+        { UC_CALLER_CATEGORY_NATIONAL_OPERATOR_CALL, "National operator call" },
+        { UC_CALLER_CATEGORY_NATIONAL_DATA_CALL, "National data call" },
+        { UC_CALLER_CATEGORY_NATIONAL_PAYPHONE_CALL, "National payphone call" },
+        { UC_CALLER_CATEGORY_INTERNATIONAL_SUBSCRIBER_CALL, "International subscriber call" },
+        { UC_CALLER_CATEGORY_INTERNATIONAL_PRIORITY_SUBSCRIBER_CALL, "International priority subscriber call" },
+        { UC_CALLER_CATEGORY_INTERNATIONAL_OPERATOR_CALL, "International operator call" },
+        { UC_CALLER_CATEGORY_INTERNATIONAL_DATA_CALL, "International data call" },
+    };
+    return code2str(code, codes, sizeof(codes)/sizeof(codes[0]));
+}
+/*- End of function --------------------------------------------------------*/
+
 void uc_dump_callparms(uc_t *uc, uc_callparms_t *callparms, int verbose)
 {
     uc_message("Bearer transfer cap: %s\n", uc_trans_cap2str(callparms->bear_cap_transfer_cap));
@@ -541,8 +558,8 @@ void uc_dump_callparms(uc_t *uc, uc_callparms_t *callparms, int verbose)
                    uc_npi2str(callparms->redirecting_npi),
                    uc_pres2str(callparms->redirecting_pres));
     }
-    uc_message("Calling party category: %d\n",
-               callparms->calling_party_category);
+    uc_message("Calling party category: %s\n",
+               uc_calling_party_category2str(callparms->calling_party_category));
     uc_message("Call type: %s\n", uc_calltype2str(callparms->call_type));
 }
 /*- End of function --------------------------------------------------------*/
