@@ -1,18 +1,3 @@
-/* Set the protocol to use */
-#define PROTOCOL_CLASS      "mfcr2"
-#define PROTOCOL_VARIANT    "mx,20,4"
-#define PROTOCOL_END        UC_MODE_CPE
-
-/* Set the test numbers to use */
-#define BASE_ANI            987654321
-#define BASE_DNIS           1236
-
-/* Set the trunks to use */
-#define FIRST_TRUNK_CIRCUIT 94
-#define NUM_TRUNKS          15
-#define FIRST_OG_TRUNK      0
-#define NUM_OG_TRUNKS       2
-
 /*
  * libunicall - a library for universal call handling on both analogue and
  *              digital telephone circuits.
@@ -39,7 +24,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: testcall.c,v 1.14 2005/02/05 13:05:45 steveu Exp $
+ * $Id: testcall.c,v 1.20 2005/08/07 13:14:51 steveu Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -64,67 +49,98 @@
 #if defined(HAVE_SPANDSP_H)
 #include <tiffio.h>
 #endif
+#if defined(HAVE_SPANDSP_H)
+#include <spandsp.h>
+#endif
+
+#include "unicall.h"
 
 #define FALSE 0
 #define TRUE (!FALSE)
 
-#if defined(HAVE_SPANDSP_H)
-#include <spandsp.h>
-#endif
-#include "unicall.h"
+#define MAX_TRUNKS      240
+
+enum
+{
+    OFFERED_ACTION_ACCEPT = 0,
+    OFFERED_ACTION_ANSWER,
+    OFFERED_ACTION_BUSY,
+    OFFERED_ACTION_UNASSIGNED,
+    OFFERED_ACTION_CONGESTED,
+    OFFERED_ACTION_OOS,
+    OFFERED_ACTION_RANDOM,
+    OFFERED_ACTION_END
+};
 
 struct
 {
+    char *protocol_class;
+    char *protocol_variant;
+    int protocol_end;
+    int chan_no;
+    int caller_mode;
+
+    char *tag;
+
+    char originating_number[UC_MAXPHONENUMLEN + 1];
+    char destination_number[UC_MAXPHONENUMLEN + 1];
+
     pthread_t thread;
-    int chan;
     int sig_fd;
     int fd;
     uc_call_t *call;
     uc_crn_t crn;
-    uint8_t message[100000];
-    int message_len;
-    int message_ptr;
-    int xxx;
-    int cause;
+    int block_head;
+    int action_on_offered;
+    
+    int og_call_number;
+    int ic_call_number;
+    uc_callparms_t call_parms;
 
 #if defined(HAVE_SPANDSP_H)
-    dtmf_rx_state_t dtmf_state;
+    dtmf_tx_state_t dtmf_tx_state;
+    dtmf_rx_state_t dtmf_rx_state;
     char dtmf[101];
     int dtmf_ptr;
 #endif
-    
-    char *tag;
-    
-    char originating_number[32];
-    char destination_number[32];
 
-    int caller_mode;
+    int silent_samples;
     
     time_t last_call;
-} chan_stuff[120];
+} chan_stuff[MAX_TRUNKS];
 
 void channel_read_file(uc_t *uc, int chan, void *user_data, uint8_t *buf, int len);
 int channel_write_file(uc_t *uc, int chan, void *user_data, uint8_t *buf, int max_len);
 int channel_error(uc_t *uc, int chan, void *user_data, int cause);
 int signaling_error(uc_t *uc, void *user_data, int cause);
 
-void channel_read_file(uc_t *uc, int chan, void *user_data, uint8_t *buf, int len)
+void channel_read_file(uc_t *uc, int ch, void *user_data, uint8_t *buf, int len)
 {
+    int16_t pcm_buf[1024];
     int i;
     int xlen;
-    int16_t pcm_buf[1024];
     char *s;
-
-    /* This routine is pretty useless for anything but illustration */
+    char *t;
+    char *u;
+    char *v;
+    char *w;
+    char originating_number[UC_MAXPHONENUMLEN + 1];
+    char destination_number[UC_MAXPHONENUMLEN + 1];
+    int call_number;
+    int chan;
+    int hits;
+    int ok;
     
+    chan = (intptr_t) user_data;
     for (i = 0;  i < len;  i++)
         pcm_buf[i] = alaw_to_linear(buf[i]);
     /*endfor*/
+
 #if defined(HAVE_SPANDSP_H)
-    dtmf_rx(&chan_stuff[chan].dtmf_state, pcm_buf, len);
-    xlen = dtmf_get(&chan_stuff[chan].dtmf_state,
-                    chan_stuff[chan].dtmf + chan_stuff[chan].dtmf_ptr,
-                    100 - chan_stuff[chan].dtmf_ptr);
+    dtmf_rx(&chan_stuff[chan].dtmf_rx_state, pcm_buf, len);
+    xlen = dtmf_rx_get(&chan_stuff[chan].dtmf_rx_state,
+                       chan_stuff[chan].dtmf + chan_stuff[chan].dtmf_ptr,
+                       100 - chan_stuff[chan].dtmf_ptr);
     if (xlen > 0)
     {
     	s = chan_stuff[chan].dtmf + chan_stuff[chan].dtmf_ptr;
@@ -132,62 +148,126 @@ void channel_read_file(uc_t *uc, int chan, void *user_data, uint8_t *buf, int le
         {
             if (*s == '#')
             {
+                printf("%s Received DTMF '%s'\n", chan_stuff[chan].tag, chan_stuff[chan].dtmf);
+                ok = FALSE;
+                call_number = -1;
+                if ((t = strchr(chan_stuff[chan].dtmf, '*')))
+                {
+                    *t++ = '\0';
+                    if ((u = strchr(t, '*')))
+                    {
+                        *u++ = '\0';
+                        if ((v = strchr(u, '*')))
+                        {
+                            *v++ = '\0';
+                            if ((w = strchr(v, '*')))
+                            {
+                                *w++ = '\0';
+                                sscanf(t, "%d", &call_number);
+                                if (strcmp(u, chan_stuff[chan].call_parms.originating_number) == 0
+                                    &&
+                                    strcmp(v, chan_stuff[chan].call_parms.destination_number) == 0
+                                    &&
+                                    call_number == chan_stuff[chan].ic_call_number)
+                                {
+                                    ok = TRUE;
+                                }
+                            }
+                        }
+                    }
+                }
+                if (!ok)
+                {
+                    printf("%s Received DTMF string incorrect %d %d '%s' '%s' '%s' '%s'\n",
+                           chan_stuff[chan].tag,
+                           call_number,
+                           chan_stuff[chan].ic_call_number,
+                           u,
+                           chan_stuff[chan].call_parms.originating_number,
+                           v,
+                           chan_stuff[chan].call_parms.destination_number);
+                }
+                chan_stuff[chan].dtmf_ptr = 0;
+                xlen = 0;
+                dtmf_tx_put(&chan_stuff[chan].dtmf_tx_state, "A");
+                break;
+            }
+            else if (*s == 'A')
+            {
+                /* We can disconnect now */
+                printf("%s Received DTMF '%s'\n", chan_stuff[chan].tag, chan_stuff[chan].dtmf);
                 uc_set_channel_read_callback(uc, 0, NULL, NULL);
                 uc_set_channel_write_callback(uc, 0, NULL, NULL);
                 if (uc_call_control(uc, UC_OP_DROPCALL, chan_stuff[chan].crn, (void *) UC_CAUSE_NORMAL_CLEARING))
             	    printf("A Drop Call failed\n");
                 /*endif*/
-                break;
+    	    	chan_stuff[chan].dtmf_ptr = 0;
+                xlen = 0;
             }
-            /*endif*/
+	    /*endif*/
             s++;
         }
         /*endwhile*/
-    	printf("Got '%s'\n", chan_stuff[chan].dtmf);
         chan_stuff[chan].dtmf_ptr += xlen;
     }
     /*endif*/
 #endif
 }
+/*- End of function --------------------------------------------------------*/
 
-int channel_write_file(uc_t *uc, int chan, void *user_data, uint8_t *buf, int max_len)
+int channel_write_file(uc_t *uc, int ch, void *user_data, uint8_t *buf, int max_len)
 {
+    int16_t pcm_buf[1024];
+    int i;
     int len;
-    int ret;
-
-    chan = (int) user_data;
-    /* This routine is pretty useless for anything but illustration */
-    if (chan_stuff[chan].message_ptr >= chan_stuff[chan].message_len)
+    int chan;
+    static int fd = -1;
+    
+    chan = (intptr_t) user_data;
+#if defined(HAVE_SPANDSP_H)
+    if (chan_stuff[chan].silent_samples > 0)
     {
-        //close(user_data);
-        uc_set_channel_read_callback(uc, 0, NULL, NULL);
-        uc_set_channel_write_callback(uc, 0, NULL, NULL);
-        if ((ret = uc_call_control(uc, UC_OP_DROPCALL, chan_stuff[chan].crn, (void *) UC_CAUSE_NORMAL_CLEARING)))
-            printf("B Drop Call failed - %d\n", ret);
-        /*endif*/
-        return 0;
+        for (i = 0;  i < max_len;  i++)
+            buf[i] = linear_to_alaw(0);
+        /*endfor*/
+        chan_stuff[chan].silent_samples -= max_len;
+        len = max_len;
+    }
+    else
+    {
+        len = dtmf_tx(&chan_stuff[chan].dtmf_tx_state, pcm_buf, max_len);
+        for (i = 0;  i < len;  i++)
+            buf[i] = linear_to_alaw(pcm_buf[i]);
+        /*endfor*/
     }
     /*endif*/
-    len = max_len;
-    if (len > chan_stuff[chan].message_len - chan_stuff[chan].message_ptr)
-        len = chan_stuff[chan].message_len - chan_stuff[chan].message_ptr;
-    /*endif*/
-    memcpy(buf, chan_stuff[chan].message + chan_stuff[chan].message_ptr, len);
-    chan_stuff[chan].message_ptr += len;
+#else
+    len = 0;
+#endif
+if (chan == 0)
+{
+    if (fd < 0)
+        fd = open("txsound.au", O_WRONLY  | O_CREAT | O_TRUNC, 0666);
+    if (fd >= 0)
+        write(fd, buf, len);
+}
     return len;
 }
+/*- End of function --------------------------------------------------------*/
 
 int channel_error(uc_t *uc, int chan, void *user_data, int cause)
 {
     printf("Error %d\n", cause);
     return  0;
 }
+/*- End of function --------------------------------------------------------*/
 
 int signaling_error(uc_t *uc, void *user_data, int cause)
 {
     printf("Error %d\n", cause);
     return  0;
 }
+/*- End of function --------------------------------------------------------*/
 
 static void initiate_call(uc_t *uc, int chan, uc_event_t *e)
 {
@@ -195,28 +275,33 @@ static void initiate_call(uc_t *uc, int chan, uc_event_t *e)
     uc_callparms_t *callparms;
     int ret;
 
-    printf ("Initiating call\n");
+    printf ("%s Initiating call\n", chan_stuff[chan].tag);
     if ((callparms = uc_new_callparms(NULL)) == NULL)
         return;
     /*endif*/
+    strcpy(chan_stuff[chan].call_parms.originating_number, chan_stuff[chan].originating_number);
+    strcpy(chan_stuff[chan].call_parms.destination_number, chan_stuff[chan].destination_number);
     uc_callparm_originating_number(callparms, chan_stuff[chan].originating_number);
     uc_callparm_destination_number(callparms, chan_stuff[chan].destination_number);
     makecall.callparms = callparms;
     makecall.crn = 0;
+    chan_stuff[chan].og_call_number++;
     if ((ret = uc_call_control(uc, UC_OP_MAKECALL, 0, (void *) &makecall)) != UC_RET_OK)
-        fprintf(stderr, "Make Call failed - %d\n", ret);
+        fprintf(stderr, "Make call failed - %d\n", ret);
     /*endif*/
     chan_stuff[chan].crn = makecall.crn;
     free(callparms);
 }
+/*- End of function --------------------------------------------------------*/
 
 static void handle_uc_event(uc_t *uc, void *user_data, uc_event_t *e)
 {
-    int file;
     time_t now;
     int chan;
+    int on_offered;
+    char buf[80];
 
-    chan = (int) user_data;
+    chan = (intptr_t) user_data;
     switch (e->e)
     {
     case UC_EVENT_DEVICEFAIL:
@@ -236,13 +321,13 @@ static void handle_uc_event(uc_t *uc, void *user_data, uc_event_t *e)
     case UC_EVENT_FARBLOCKED:
         printf("%s -- %s! :-(\n", chan_stuff[chan].tag, uc_event2str(e->e));
         fprintf(stderr, "%s -- %s! :-(\n", chan_stuff[chan].tag, uc_event2str(e->e));
-        chan_stuff[chan].xxx &= ~1;
+        chan_stuff[chan].block_head &= ~1;
         break;
     case UC_EVENT_FARUNBLOCKED:
         printf("%s -- %s! :-)\n", chan_stuff[chan].tag, uc_event2str(e->e));
         fprintf(stderr, "%s -- %s! :-)\n", chan_stuff[chan].tag, uc_event2str(e->e));
-        chan_stuff[chan].xxx |= 1;
-        if (chan_stuff[chan].xxx == 3)
+        chan_stuff[chan].block_head |= 1;
+        if (chan_stuff[chan].block_head == 3)
         {
             if (chan_stuff[chan].caller_mode)
                 initiate_call(uc, chan, e);
@@ -253,13 +338,13 @@ static void handle_uc_event(uc_t *uc, void *user_data, uc_event_t *e)
     case UC_EVENT_LOCALBLOCKED:
         printf("%s -- %s! :-(\n", chan_stuff[chan].tag, uc_event2str(e->e));
         fprintf(stderr, "%s -- %s! :-(\n", chan_stuff[chan].tag, uc_event2str(e->e));
-        chan_stuff[chan].xxx &= ~2;
+        chan_stuff[chan].block_head &= ~2;
         break;
     case UC_EVENT_LOCALUNBLOCKED:
         printf("%s -- %s! :-)\n", chan_stuff[chan].tag, uc_event2str(e->e));
         fprintf(stderr, "%s -- %s! :-)\n", chan_stuff[chan].tag, uc_event2str(e->e));
-        chan_stuff[chan].xxx |= 2;
-        if (chan_stuff[chan].xxx == 3)
+        chan_stuff[chan].block_head |= 2;
+        if (chan_stuff[chan].block_head == 3)
         {
             if (chan_stuff[chan].caller_mode)
                 initiate_call(uc, chan, e);
@@ -287,6 +372,7 @@ static void handle_uc_event(uc_t *uc, void *user_data, uc_event_t *e)
                e->gen.channel, e->gen.crn);
         fprintf(stderr, "%s -- %s on channel %d, CRN %d\n", chan_stuff[chan].tag, uc_event2str(e->e), e->gen.channel, e->gen.crn);
         chan_stuff[chan].crn = e->gen.crn;
+        chan_stuff[chan].ic_call_number++;
         break;
     case UC_EVENT_MOREDIGITS:
         printf("%s -- %s on channel %d\n", chan_stuff[chan].tag, uc_event2str(e->e), e->gen.channel);
@@ -344,42 +430,51 @@ static void handle_uc_event(uc_t *uc, void *user_data, uc_event_t *e)
                 e->offered.crn,
                 e->offered.parms.originating_number,
                 e->offered.parms.destination_number);
+        memcpy(&chan_stuff[chan].call_parms, &e->offered.parms, sizeof(chan_stuff[chan].call_parms));
         if (!chan_stuff[chan].caller_mode)
         {
             time(&now);
-            switch (chan_stuff[chan].cause)
+            on_offered = chan_stuff[chan].action_on_offered;
+            if (on_offered == OFFERED_ACTION_RANDOM)
             {
-            case 0:
+                do
+                    on_offered = rand() & 0x7;
+                while (on_offered >= OFFERED_ACTION_END);
+            }
+            /*endif*/
+            switch (on_offered)
+            {
+            case OFFERED_ACTION_ACCEPT:
                 fprintf(stderr, "%s -- +++ ACCEPT (%d)\n", chan_stuff[chan].tag, now - chan_stuff[chan].last_call);
                 if (uc_call_control(uc, UC_OP_ACCEPTCALL, e->offered.crn, NULL))
                     fprintf(stderr, "Accept Call failed\n");
                 /*endif*/
                 break;
-            case 1:
+            case OFFERED_ACTION_ANSWER:
                 fprintf(stderr, "%s -- +++ ANSWER (%d)\n", chan_stuff[chan].tag, now - chan_stuff[chan].last_call);
                 if (uc_call_control(uc, UC_OP_ANSWERCALL, e->offered.crn, NULL))
                     fprintf(stderr, "Answer Call failed\n");
                 /*endif*/
                 break;
-            case 2:
+            case OFFERED_ACTION_BUSY:
                 fprintf(stderr, "%s -- +++ BUSY (%d)\n", chan_stuff[chan].tag, now - chan_stuff[chan].last_call);
                 if (uc_call_control(uc, UC_OP_DROPCALL, e->offered.crn, (void *) UC_CAUSE_USER_BUSY))
                     fprintf(stderr, "E Drop Call failed\n");
                 /*endif*/
                 break;
-            case 3:
+            case OFFERED_ACTION_UNASSIGNED:
                 fprintf(stderr, "%s -- +++ UNASSIGNED (%d)\n", chan_stuff[chan].tag, now - chan_stuff[chan].last_call);
                 if (uc_call_control(uc, UC_OP_DROPCALL, e->offered.crn, (void *) UC_CAUSE_UNASSIGNED_NUMBER))
                     fprintf(stderr, "F Drop Call failed\n");
                 /*endif*/
                 break;
-            case 4:
+            case OFFERED_ACTION_CONGESTED:
                 fprintf(stderr, "%s -- +++ CONGESTED (%d)\n", chan_stuff[chan].tag, now - chan_stuff[chan].last_call);
                 if (uc_call_control(uc, UC_OP_DROPCALL, e->offered.crn, (void *) UC_CAUSE_NETWORK_CONGESTION))
                     fprintf(stderr, "G Drop Call failed\n");
                 /*endif*/
                 break;
-            case 5:
+            case OFFERED_ACTION_OOS:
                 fprintf(stderr, "%s -- +++ OUT OF SERVICE (%d)\n", chan_stuff[chan].tag, now - chan_stuff[chan].last_call);
                 if (uc_call_control(uc, UC_OP_DROPCALL, e->offered.crn, (void *) UC_CAUSE_DEST_OUT_OF_ORDER))
                     fprintf(stderr, "H Drop Call failed\n");
@@ -388,44 +483,38 @@ static void handle_uc_event(uc_t *uc, void *user_data, uc_event_t *e)
             }
             /*endswitch*/
             chan_stuff[chan].last_call = now;
-            if (++chan_stuff[chan].cause > 5)
-                chan_stuff[chan].cause = 0;
-            /*endif*/
         }
         /*endif*/
         break;
     case UC_EVENT_ANSWERED:
         printf("%s -- %s on channel %d\n", chan_stuff[chan].tag, uc_event2str(e->e), e->gen.channel);
         fprintf(stderr, "%s -- %s on channel %d\n", chan_stuff[chan].tag, uc_event2str(e->e), e->gen.channel);
-        uc_set_channel_read_callback(uc, 0, channel_read_file, (void *) chan);
-        file = open("audio", O_RDONLY);
-        if (file < 0)
-        {
-            /* No audio file */
-            printf("%s -- Outgoing audio file not found\n", chan_stuff[chan].tag);
-            fprintf(stderr, "%s -- Outgoing audio file not found\n", chan_stuff[chan].tag);
-            chan_stuff[chan].message_len = 0;
-            chan_stuff[chan].message_ptr = 0;
-        }
-        else
-        {
-            chan_stuff[chan].message_len = read(file, chan_stuff[chan].message, 100000);
-            chan_stuff[chan].message_ptr = 0;
-            printf("%s -- %d bytes of audio to send\n", chan_stuff[chan].tag, chan_stuff[chan].message_len);
-            fprintf(stderr, "%s -- %d bytes of audio to send\n", chan_stuff[chan].tag, chan_stuff[chan].message_len);
-            close(file);
-        }
-        uc_set_channel_write_callback(uc, 0, channel_write_file, (void *) chan);
+        uc_set_channel_read_callback(uc, 0, channel_read_file, (void *) (intptr_t) chan);
+    	chan_stuff[chan].og_call_number++;
 #if defined(HAVE_SPANDSP_H)
-        dtmf_rx_init(&chan_stuff[chan].dtmf_state, NULL, NULL);
+        dtmf_rx_init(&chan_stuff[chan].dtmf_rx_state, NULL, NULL);
+        dtmf_tx_init(&chan_stuff[chan].dtmf_tx_state);
+        chan_stuff[chan].silent_samples = 4000;
+        sprintf(buf, "*%08d*%s*%s*#", chan_stuff[chan].og_call_number, chan_stuff[chan].originating_number, chan_stuff[chan].destination_number);
+        printf("%s -- '%s'\n", chan_stuff[chan].tag, buf);
+        dtmf_tx_put(&chan_stuff[chan].dtmf_tx_state, buf);
 #endif
+        uc_set_channel_write_callback(uc, 0, channel_write_file, (void *) (intptr_t) chan);
         break;
     case UC_EVENT_CONNECTED:
         printf("%s -- %s on channel %d\n", chan_stuff[chan].tag, uc_event2str(e->e), e->gen.channel);
         fprintf(stderr, "%s -- %s on channel %d\n", chan_stuff[chan].tag, uc_event2str(e->e), e->gen.channel);
-        if (uc_call_control(uc, UC_OP_DROPCALL, e->offered.crn, (void *) UC_CAUSE_NORMAL_CLEARING))
-            printf ("I Drop Call failed\n");
-        /*endif*/
+        uc_set_channel_read_callback(uc, 0, channel_read_file, (void *) (intptr_t) chan);
+    	chan_stuff[chan].ic_call_number++;
+#if defined(HAVE_SPANDSP_H)
+        dtmf_rx_init(&chan_stuff[chan].dtmf_rx_state, NULL, NULL);
+        dtmf_tx_init(&chan_stuff[chan].dtmf_tx_state);
+        chan_stuff[chan].silent_samples = 4000;
+        sprintf(buf, "*%08d*%s*%s*#", chan_stuff[chan].og_call_number, chan_stuff[chan].originating_number, chan_stuff[chan].destination_number);
+        printf("%s -- '%s'\n", chan_stuff[chan].tag, buf);
+        dtmf_tx_put(&chan_stuff[chan].dtmf_tx_state, buf);
+#endif
+        uc_set_channel_write_callback(uc, 0, channel_write_file, (void *) (intptr_t) chan);
         break;
     default:
         printf ("%s -- %s\n", chan_stuff[chan].tag, uc_event2str(e->e));
@@ -434,6 +523,7 @@ static void handle_uc_event(uc_t *uc, void *user_data, uc_event_t *e)
     }
     /*endswitch*/
 }
+/*- End of function --------------------------------------------------------*/
 
 static void *run_uc(void *arg)
 {
@@ -447,21 +537,24 @@ static void *run_uc(void *arg)
     int res;
     int dfd;
     int chan;
-    
-    chan = *((int *) arg);
+    int logging_level;
+
+    chan = (int) (intptr_t) arg;
 
     dfd = chan_stuff[chan].fd;
-    uc = uc_new(dfd, dfd, PROTOCOL_CLASS, PROTOCOL_VARIANT, PROTOCOL_END, 1);
+    uc = uc_new(dfd, dfd, chan_stuff[chan].protocol_class, chan_stuff[chan].protocol_variant, chan_stuff[chan].protocol_end, 1);
     if (uc == NULL)
     {
         fprintf(stderr, "Unable to create instance\n");
         return NULL;
     }
     /*endif*/
-    uc_set_signaling_callback(uc, handle_uc_event, (void *) chan);
-    uc_set_signaling_error_callback(uc, signaling_error, (void *) chan);
-    uc_set_channel_error_callback(uc, 0, channel_error, (void *) chan);
-    uc_set_logging(uc, 0x7FFFFFFF, 1, chan_stuff[chan].tag);
+    uc_set_signaling_callback(uc, handle_uc_event, (void *) (intptr_t) chan);
+    uc_set_signaling_error_callback(uc, signaling_error, (void *) (intptr_t) chan);
+    uc_set_channel_error_callback(uc, 0, channel_error, (void *) (intptr_t) chan);
+    logging_level = 2 & UC_LOG_SEVERITY_MASK;
+    logging_level |= (UC_LOG_SHOW_TAG | UC_LOG_SHOW_PROTOCOL);
+    uc_set_logging(uc, logging_level, 1, chan_stuff[chan].tag);
     uc_call_control(uc, UC_OP_UNBLOCK, 0, (void *) -1);
     for (;;)
     {
@@ -500,7 +593,7 @@ static void *run_uc(void *arg)
             while ((e = uc_check_event(uc)))
             {
                 printf("Non-callback signaling event\n");
-                handle_uc_event(uc, (void *) chan, e);
+                handle_uc_event(uc, (void *) (intptr_t) chan, e);
             }
             /*endwhile*/
         }
@@ -513,162 +606,183 @@ static void *run_uc(void *arg)
     /*endfor*/
     return NULL;
 }
+/*- End of function --------------------------------------------------------*/
+
+void increment_digit_string(char *buf)
+{
+    int i;
+    
+    i = strlen(buf);
+    while (i > 0)
+    {
+        i--;
+        if (buf[i] < '9')
+        {
+            buf[i]++;
+            return;
+        }
+        buf[i] = '0';
+    }
+}
+/*- End of function --------------------------------------------------------*/
+
+int get_configuration(void)
+{
+    FILE *conf;
+    char buf[133];
+    char buf2[133];
+    int value;
+    int value2;
+    int chan;
+    int i;
+    char originating_number[UC_MAXPHONENUMLEN + 1];
+    char destination_number[UC_MAXPHONENUMLEN + 1];
+    char protocol_class[80];
+    char protocol_variant[80];
+    int protocol_end;
+    int caller_mode;
+    int action_on_offered;
+    
+    chan = 0;
+    if ((conf = fopen("testcall.conf", "r")) == NULL)
+        return -1;
+    while (fgets(buf, 132, conf))
+    {
+        if (buf[0] == '#')
+            continue;
+        /*endif*/
+        if (sscanf(buf, "protocol-class %s", protocol_class) == 1)
+        {
+        }
+        else if (sscanf(buf, "protocol-variant %s", protocol_variant) == 1)
+        {
+        }
+        else if (sscanf(buf, "protocol-end %s", buf2) == 1)
+        {
+            protocol_end = (strcmp(buf2, "cpe") == 0)  ?  UC_MODE_CPE  :  UC_MODE_CO;
+        }
+        else if (sscanf(buf, "caller %s", buf2) == 1)
+        {
+            caller_mode = (strcmp(buf2, "yes") == 0);
+        }
+        else if (sscanf(buf, "originating-no %s", originating_number) == 1)
+        {
+        }
+        else if (sscanf(buf, "destination-no %s", destination_number) == 1)
+        {
+        }
+        else if (sscanf(buf, "circuits %d-%d", &value, &value2) == 2)
+        {
+            for (i = value;  i <= value2;  i++)
+            {
+                chan_stuff[chan].chan_no = i;
+                chan_stuff[chan].protocol_class = strdup(protocol_class);
+                chan_stuff[chan].protocol_variant = strdup(protocol_variant);
+                chan_stuff[chan].protocol_end = protocol_end;
+                chan_stuff[chan].caller_mode = caller_mode;
+                strcpy(chan_stuff[chan].originating_number, originating_number);
+                strcpy(chan_stuff[chan].destination_number, destination_number);
+                increment_digit_string(originating_number);
+                increment_digit_string(destination_number);
+                chan_stuff[chan].action_on_offered = action_on_offered;
+                fprintf(stderr,
+                        "Chan %d, class '%s', variant '%s', end %d, caller %d, from '%s' to '%s'\n",
+                        chan_stuff[chan].chan_no,
+                        chan_stuff[chan].protocol_class,
+                        chan_stuff[chan].protocol_variant,
+                        chan_stuff[chan].protocol_end,
+                        chan_stuff[chan].caller_mode,
+                        chan_stuff[chan].originating_number,
+                        chan_stuff[chan].destination_number);
+                chan++;
+            }
+        }
+        else if (sscanf(buf, "on-offered %s", buf2) == 1)
+        {
+            if (strcmp(buf2, "accept") == 0)
+                action_on_offered = OFFERED_ACTION_ACCEPT;
+            else if (strcmp(buf2, "answer") == 0)
+                action_on_offered = OFFERED_ACTION_ANSWER;
+            else if (strcmp(buf2, "busy") == 0)
+                action_on_offered = OFFERED_ACTION_BUSY;
+            else if (strcmp(buf2, "unassigned") == 0)
+                action_on_offered = OFFERED_ACTION_UNASSIGNED;
+            else if (strcmp(buf2, "congested") == 0)
+                action_on_offered = OFFERED_ACTION_CONGESTED;
+            else if (strcmp(buf2, "oos") == 0)
+                action_on_offered = OFFERED_ACTION_OOS;
+            else if (strcmp(buf2, "random") == 0)
+                action_on_offered = OFFERED_ACTION_RANDOM;
+            else
+                action_on_offered = OFFERED_ACTION_ACCEPT;
+        }
+        else
+        {
+            fprintf(stderr, "Invalid configuration file entry '%s'\n", buf);
+            fclose(conf);
+            return -1;
+        }
+    }
+    fclose(conf);
+    return 0;
+}
+/*- End of function --------------------------------------------------------*/
 
 int main(int argc, char *argv[]) 
 {
     pthread_attr_t attr;
-    struct zt_bufferinfo b;
-    struct zt_gains g;
-    struct zt_spaninfo si;
     int chan;
-    int chanx;
     char dev_name[20];
     int j;
     int x;
     ZT_PARAMS p;
 
-    uc_start();    
-    pthread_attr_init(&attr);
-    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
     if (argc < 1)
     {
         fprintf(stderr, "Usage: testcall [call]\n");
         exit(1);
     }
     /*endif*/
-    for (chan = 0;  chan < NUM_TRUNKS;  chan++)
+    if (get_configuration() < 0)
     {
-        chan_stuff[chan].sig_fd = open("/dev/zap/channel", O_RDWR | O_NONBLOCK);
-        if (chan_stuff[chan].sig_fd < 0)
+        fprintf(stderr, "Can't read the configuration file\n");
+        exit(1);
+    }
+    /*endif*/
+    uc_start();    
+    pthread_attr_init(&attr);
+    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
+    /* Create and start a thread for each defined channel */
+    for (chan = 0;  chan < MAX_TRUNKS;  chan++)
+    {
+        if (chan_stuff[chan].protocol_class == NULL)
+            continue;
+        /*endif*/
+        sprintf(dev_name, "%d", chan_stuff[chan].chan_no);
+        if ((chan_stuff[chan].sig_fd = uc_channel_open(chan_stuff[chan].protocol_class, chan_stuff[chan].protocol_variant, chan_stuff[chan].protocol_end, dev_name)) < 0)
         {
             fprintf(stderr, "Failed to open channel: %s\n", strerror(errno));
             exit(1);
         }
         /*endif*/
         chan_stuff[chan].fd = chan_stuff[chan].sig_fd;
-        
-        /* Allow for the missing channel at TS16 */
-        if (chan < 15)
-            chanx = chan + FIRST_TRUNK_CIRCUIT;
-        else if (chan < 45)
-            chanx = chan + FIRST_TRUNK_CIRCUIT + 1;
-        else
-            chanx = chan + FIRST_TRUNK_CIRCUIT + 2;
-        /*endif*/
-        if (ioctl(chan_stuff[chan].fd, ZT_SPECIFY, &chanx))
-        {
-            fprintf(stderr, "Failed to specify channel %d: %s\n", chanx, strerror(errno));
-            exit(1);
-        }
-        /*endif*/
-        if (ioctl(chan_stuff[chan].fd, ZT_GET_BUFINFO, &b))
-        {
-            fprintf(stderr, "Unable to get buffer info on channel %d: %s\n", chanx, strerror(errno));
-            exit(1);
-        }
-        /*endif*/
-        printf ("%d %d %d %d %d %d\n",
-                b.rxbufpolicy,
-                b.txbufpolicy,
-                b.numbufs,
-                b.bufsize,
-                b.readbufs,
-                b.writebufs);
-        b.rxbufpolicy = ZT_POLICY_IMMEDIATE;
-        b.txbufpolicy = ZT_POLICY_IMMEDIATE;
-        b.numbufs = 4;
-        b.bufsize = 160;
-        if (ioctl(chan_stuff[chan].fd, ZT_SET_BUFINFO, &b))
-        {
-            fprintf(stderr, "Unable to set buffer info on channel %d: %s\n", chanx, strerror(errno));
-            exit(1);
-        }
-        /*endif*/
-        if (ioctl(chan_stuff[chan].fd, ZT_GET_BUFINFO, &b))
-        {
-            fprintf(stderr, "Unable to get buffer info on channel %d: %s\n", chanx, strerror(errno));
-            exit(1);
-        }
-        /*endif*/
-        printf("%d %d %d %d %d %d\n",
-               b.rxbufpolicy,
-               b.txbufpolicy,
-    	       b.numbufs,
-               b.bufsize,
-               b.readbufs,
-               b.writebufs);
-        memset(&p, 0, sizeof(p));
-        if (ioctl(chan_stuff[chan].fd, ZT_GET_PARAMS, &p))
-        {
-            fprintf(stderr, "Unable to get params on channel %d: %s\n", chanx, strerror(errno));
-            exit(1);
-        }
-        /*endif*/
-        memset(&si, 0, sizeof(si));
-        if (ioctl(chan_stuff[chan].fd, ZT_SPANSTAT, &si))
-        {
-            fprintf(stderr, "Unable to get spanstat on channel %d: %s\n", chanx, strerror(errno));
-            exit(1);
-        }
-        /*endif*/
-        x = 160;
-        if (ioctl(chan_stuff[chan].fd, ZT_SET_BLOCKSIZE, &x))
-        {
-            fprintf(stderr, "Unable to set block size on channel %d: %s\n", chanx, strerror(errno));
-            exit(1);
-        }
-        /*endif*/
-#if 0
-        x = ZT_LAW_ALAW;
-        if (ioctl(chan_stuff[chan].fd, ZT_SETLAW, &x))
-        {
-            fprintf(stderr, "Unable to set law on channel %d: %s\n", chanx, strerror(errno));
-            exit(1);
-        }
-        /*endif*/
-        x = 1;
-        if (ioctl(chan_stuff[chan].fd, ZT_SETLINEAR, &x))
-        {
-            fprintf(stderr, "Unable to set linear on channel %d: %s\n", chanx, strerror(errno));
-            exit(1);
-        }
-        /*endif*/
-#endif
-        g.chan = 0;
-        for (j = 0;  j < 256;  j++)
-        {
-            g.rxgain[j] = j;
-            g.txgain[j] = j;
-        }
-        if (ioctl(chan_stuff[chan].fd, ZT_SETGAINS, &g))
-        {
-            fprintf(stderr, "Unable to set gains on channel %d: %s\n", chanx, strerror(errno));
-            exit(1);
-        }
-        /*endif*/
-        if (chan >= FIRST_OG_TRUNK  &&  chan < FIRST_OG_TRUNK + NUM_OG_TRUNKS)
-            chan_stuff[chan].caller_mode = TRUE;
-        else
-            chan_stuff[chan].caller_mode = FALSE;
-        /*endif*/
-        chan_stuff[chan].chan = chan;
-        sprintf(dev_name, "Chan %3d:", chanx);
+
+        sprintf(dev_name, "Chan %3d:", chan_stuff[chan].chan_no);
         chan_stuff[chan].tag = strdup(dev_name);
-        sprintf(chan_stuff[chan].originating_number, "%d", BASE_ANI + chan);
-        sprintf(chan_stuff[chan].destination_number, "%d", BASE_DNIS + chan);
         printf("Thread for channel %d\n", chan);
-        if (pthread_create(&chan_stuff[chan].thread, &attr, run_uc, &chan_stuff[chan].chan))
+        if (pthread_create(&chan_stuff[chan].thread, &attr, run_uc, (void *) (intptr_t) chan))
             exit(2);
         /*endif*/
     }
     /*endfor*/
     for (;;)
     {
-        sleep(5);
+        usleep(5000000);
         printf("Main thread\n");
     }
     /*endfor*/
-    exit(0);
 
     return 0;
 }
+/*- End of function --------------------------------------------------------*/
+/*- End of file ------------------------------------------------------------*/
