@@ -24,7 +24,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: unicall.c,v 1.11 2004/10/27 15:34:42 steveu Exp $
+ * $Id: unicall.c,v 1.18 2005/07/11 13:22:24 steveu Exp $
  */
 
 /*! \file unicall.h unicall-private.h */
@@ -240,16 +240,11 @@ int uc_str2protocol_variant(int protocol_class, char *variant)
     s = protocol_specs[protocol_class].protocol->variants;
     for (i = 1;  *s;  i++)
     {
-        for (len = 0;  s[len]  &&  s[len] != ',';  len++)
-            /* dummy loop */;
-        /*endfor*/
-        if (strncmp (variant, s, len) == 0)
+        len = strlen(s);
+        if (strncmp(variant, s, len) == 0)
             return  i;
         /*endif*/
-        s += len;
-        if (*s == ',')
-            s++;
-        /*endif*/
+        s += (len + 1);
     }
     /*endfor*/
     return  -1;
@@ -282,16 +277,9 @@ const char *uc_protocol_variant2str(int protocol_class, int protocol_variant)
         return  NULL;
     /*endif*/
     s = protocol_specs[protocol_class].protocol->variant_descriptions;
-    for (  ;  *s  &&  protocol_variant;  protocol_variant--)
-    {
-        while (*s  &&  *s != ',')
-            s++;
-        /*endwhile*/
-        if (*s == ',')
-            s++;
-        /*endif*/
-    }
-    /*endfor*/
+    while (*s  &&  --protocol_variant)
+        s += (strlen(s) + 1);
+    /*endwhile*/
     return  s;
 }
 /*- End of function --------------------------------------------------------*/
@@ -343,24 +331,6 @@ int uc_set_channel_error_callback(uc_t *uc, int ch, uc_channel_error_callback_t
 }
 /*- End of function --------------------------------------------------------*/
 
-int uc_get_native_codec(uc_t *uc, int ch)
-{
-    if (ch >= uc->num_channels)
-        return -1;
-    /*endif*/
-    return  uc->chan[ch].native_codec;
-}
-/*- End of function --------------------------------------------------------*/
-
-int uc_get_api_codec(uc_t *uc, int ch)
-{
-    if (ch >= uc->num_channels)
-        return -1;
-    /*endif*/
-    return  uc->chan[ch].api_codec;
-}
-/*- End of function --------------------------------------------------------*/
-
 char *uc_error_message(uc_t *uc, int code)
 {
     uc_call_t *call;
@@ -398,6 +368,74 @@ int uc_get_device_handles(uc_t *uc, int ch, int *fe, int *be)
 }
 /*- End of function --------------------------------------------------------*/
 
+int uc_channel_open(const char *protocol,
+                    const char *variant,
+                    int mode,
+                    char *dev)
+{
+    uc_t *uc;
+    int protocol_class;
+
+    if ((protocol_class = str2protocolclass(protocol)) < 0)
+        return -1;
+    /*endif*/
+    return  protocol_specs[protocol_class].protocol->xx_channel_open(variant, dev);
+}
+/*- End of function --------------------------------------------------------*/
+
+int uc_channel_close(const char *protocol,
+                     const char *variant,
+                     int mode,
+                     int fd)
+{
+    uc_t *uc;
+    int protocol_class;
+
+    if ((protocol_class = str2protocolclass(protocol)) < 0)
+        return -1;
+    /*endif*/
+    return  protocol_specs[protocol_class].protocol->xx_channel_close(fd);
+}
+/*- End of function --------------------------------------------------------*/
+
+int uc_channel_set_api_codec(uc_t *uc, int ch, int codec)
+{
+    if (ch >= uc->num_channels)
+        return  UC_RET_BAD_CHANNEL;
+    /*endif*/
+    if (uc->chan[ch].api_codec != codec)
+    {
+        if (uc->protocol_class < max_protocol_spec
+            &&
+            protocol_specs[uc->protocol_class].protocol)
+        {
+            return  protocol_specs[uc->protocol_class].protocol->xx_channel_set_api_codec(uc, ch, codec);
+        }
+        /*endif*/
+    }
+    /*endif*/
+    return  UC_RET_OK;
+}
+/*- End of function --------------------------------------------------------*/
+
+int uc_channel_get_native_codec(uc_t *uc, int ch)
+{
+    if (ch >= uc->num_channels)
+        return -1;
+    /*endif*/
+    return  uc->chan[ch].native_codec;
+}
+/*- End of function --------------------------------------------------------*/
+
+int uc_channel_get_api_codec(uc_t *uc, int ch)
+{
+    if (ch >= uc->num_channels)
+        return -1;
+    /*endif*/
+    return  uc->chan[ch].api_codec;
+}
+/*- End of function --------------------------------------------------------*/
+
 int uc_channel_write(uc_t *uc, int ch, uint8_t *buf, int len)
 {
     if (ch >= uc->num_channels)
@@ -430,23 +468,51 @@ int uc_channel_flush(uc_t *uc, int ch, int which)
 }
 /*- End of function --------------------------------------------------------*/
 
-int uc_set_api_codec(uc_t *uc, int ch, int codec)
+int uc_channel_gains(uc_t *uc, int ch, float rxgain, float txgain)
 {
     if (ch >= uc->num_channels)
         return  UC_RET_BAD_CHANNEL;
     /*endif*/
-    if (uc->chan[ch].api_codec != codec)
+    if (uc->protocol_class < max_protocol_spec
+        &&
+        protocol_specs[uc->protocol_class].protocol)
     {
-        if (uc->protocol_class < max_protocol_spec
-            &&
-            protocol_specs[uc->protocol_class].protocol)
-        {
-            return  protocol_specs[uc->protocol_class].protocol->xx_set_api_codec(uc, ch, codec);
-        }
-        /*endif*/
+        return  protocol_specs[uc->protocol_class].protocol->xx_channel_gains(uc, ch, rxgain, txgain);
     }
     /*endif*/
-    return  UC_RET_OK;
+    return  UC_RET_WRONG_PROTOCOL;
+}
+/*- End of function --------------------------------------------------------*/
+
+int uc_channel_echo_cancel(uc_t *uc, int ch, int op)
+{
+    if (ch >= uc->num_channels)
+        return  UC_RET_BAD_CHANNEL;
+    /*endif*/
+    if (uc->protocol_class < max_protocol_spec
+        &&
+        protocol_specs[uc->protocol_class].protocol)
+    {
+        return  protocol_specs[uc->protocol_class].protocol->xx_channel_echo_cancel(uc, ch, op);
+    }
+    /*endif*/
+    return  UC_RET_WRONG_PROTOCOL;
+}
+/*- End of function --------------------------------------------------------*/
+
+int uc_channel_switching(uc_t *uc, int ch, int op, int dest, int parms)
+{
+    if (ch >= uc->num_channels)
+        return  UC_RET_BAD_CHANNEL;
+    /*endif*/
+    if (uc->protocol_class < max_protocol_spec
+        &&
+        protocol_specs[uc->protocol_class].protocol)
+    {
+        return  protocol_specs[uc->protocol_class].protocol->xx_channel_switching(uc, ch, op, dest, parms);
+    }
+    /*endif*/
+    return  UC_RET_WRONG_PROTOCOL;
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -521,6 +587,7 @@ uc_t *uc_new(int fe,
 {
     uc_t *uc;
     int protocol_class;
+
     if ((protocol_class = str2protocolclass(protocol)) < 0)
         return  NULL;
     /*endif*/
@@ -599,52 +666,6 @@ void uc_callparm_user_rate(uc_callparms_t *parms, int rate)
 }
 /*- End of function --------------------------------------------------------*/
 
-void uc_callparm_originating_ton(uc_callparms_t *parms, int ton)
-{
-    parms->originating_ton = ton;
-}
-/*- End of function --------------------------------------------------------*/
-
-void uc_callparm_originating_npi(uc_callparms_t *parms, int npi)
-{
-    parms->originating_npi = npi;
-}
-/*- End of function --------------------------------------------------------*/
-
-void uc_callparm_originating_number(uc_callparms_t *parms, char *num)
-{
-    if (num)
-        strncpy(parms->originating_number, num, sizeof(parms->originating_number) - 1);
-    else
-        parms->originating_number[0] = '\0';
-    /*endif*/
-    parms->originating_number[sizeof(parms->originating_number) - 1] = '\0';
-}
-/*- End of function --------------------------------------------------------*/
-
-void uc_callparm_originating_sub_addr_ton(uc_callparms_t *parms, int ton)
-{
-    parms->originating_sub_addr_ton = ton;
-}
-/*- End of function --------------------------------------------------------*/
-
-void uc_callparm_originating_sub_addr_npi(uc_callparms_t *parms, int npi)
-{
-    parms->originating_sub_addr_npi = npi;
-}
-/*- End of function --------------------------------------------------------*/
-
-void uc_callparm_originating_sub_addr_number(uc_callparms_t *parms, char *num)
-{
-    if (num)
-        strncpy(parms->originating_sub_addr_number, num, sizeof(parms->originating_sub_addr_number) - 1);
-    else
-        parms->originating_sub_addr_number[0] = '\0';
-    /*endif*/
-    parms->originating_sub_addr_number[sizeof(parms->originating_sub_addr_number) - 1] = '\0';
-}
-/*- End of function --------------------------------------------------------*/
-
 void uc_callparm_destination_ton(uc_callparms_t *parms, int ton)
 {
     parms->destination_ton = ton;
@@ -691,6 +712,58 @@ void uc_callparm_destination_sub_addr_number(uc_callparms_t *parms, char *num)
 }
 /*- End of function --------------------------------------------------------*/
 
+void uc_callparm_originating_presentation(uc_callparms_t *parms, int pres)
+{
+    parms->originating_pres = pres;
+}
+/*- End of function --------------------------------------------------------*/
+
+void uc_callparm_originating_ton(uc_callparms_t *parms, int ton)
+{
+    parms->originating_ton = ton;
+}
+/*- End of function --------------------------------------------------------*/
+
+void uc_callparm_originating_npi(uc_callparms_t *parms, int npi)
+{
+    parms->originating_npi = npi;
+}
+/*- End of function --------------------------------------------------------*/
+
+void uc_callparm_originating_number(uc_callparms_t *parms, char *num)
+{
+    if (num)
+        strncpy(parms->originating_number, num, sizeof(parms->originating_number) - 1);
+    else
+        parms->originating_number[0] = '\0';
+    /*endif*/
+    parms->originating_number[sizeof(parms->originating_number) - 1] = '\0';
+}
+/*- End of function --------------------------------------------------------*/
+
+void uc_callparm_originating_sub_addr_ton(uc_callparms_t *parms, int ton)
+{
+    parms->originating_sub_addr_ton = ton;
+}
+/*- End of function --------------------------------------------------------*/
+
+void uc_callparm_originating_sub_addr_npi(uc_callparms_t *parms, int npi)
+{
+    parms->originating_sub_addr_npi = npi;
+}
+/*- End of function --------------------------------------------------------*/
+
+void uc_callparm_originating_sub_addr_number(uc_callparms_t *parms, char *num)
+{
+    if (num)
+        strncpy(parms->originating_sub_addr_number, num, sizeof(parms->originating_sub_addr_number) - 1);
+    else
+        parms->originating_sub_addr_number[0] = '\0';
+    /*endif*/
+    parms->originating_sub_addr_number[sizeof(parms->originating_sub_addr_number) - 1] = '\0';
+}
+/*- End of function --------------------------------------------------------*/
+
 void uc_callparm_redirecting_cause(uc_callparms_t *parms, int cause)
 {
     parms->redirecting_cause = cause;
@@ -726,6 +799,58 @@ void uc_callparm_redirecting_number(uc_callparms_t *parms, char *num)
 }
 /*- End of function --------------------------------------------------------*/
 
+void uc_callparm_redirecting_subaddr_ton(uc_callparms_t *parms, int ton)
+{
+    parms->redirecting_subaddr_ton = ton;
+}
+/*- End of function --------------------------------------------------------*/
+
+void uc_callparm_redirecting_subaddr_npi(uc_callparms_t *parms, int npi)
+{
+    parms->redirecting_subaddr_npi = npi;
+}
+/*- End of function --------------------------------------------------------*/
+
+void uc_callparm_redirecting_subaddr(uc_callparms_t *parms, char *num)
+{
+    if (num)
+        strncpy(parms->redirecting_subaddr, num, sizeof(parms->redirecting_number) - 1);
+    else
+        parms->redirecting_subaddr[0] = '\0';
+    /*endif*/
+    parms->redirecting_subaddr[sizeof(parms->redirecting_subaddr) - 1] = '\0';
+}
+/*- End of function --------------------------------------------------------*/
+
+void uc_callparm_original_called_number_ton(uc_callparms_t *parms, int ton)
+{
+    parms->original_called_number_ton = ton;
+}
+/*- End of function --------------------------------------------------------*/
+
+void uc_callparm_original_called_number_npi(uc_callparms_t *parms, int npi)
+{
+    parms->original_called_number_npi = npi;
+}
+/*- End of function --------------------------------------------------------*/
+
+void uc_callparm_original_called_number(uc_callparms_t *parms, char *num)
+{
+    if (num)
+        strncpy(parms->original_called_number, num, sizeof(parms->original_called_number) - 1);
+    else
+        parms->original_called_number[0] = '\0';
+    /*endif*/
+    parms->original_called_number[sizeof(parms->original_called_number) - 1] = '\0';
+}
+/*- End of function --------------------------------------------------------*/
+
+void uc_callparm_calling_party_category(uc_callparms_t *parms, int category)
+{
+    parms->calling_party_category = category;
+}
+/*- End of function --------------------------------------------------------*/
+
 void uc_set_logging(uc_t *uc, int level, int format, const char *tag)
 {
     uc->logging_level = level;
@@ -893,7 +1018,7 @@ uc_call_t *uc_findcall(uc_t *uc, int crn)
 {
     hash_HashEntry_t *entry;
 
-    if ((entry = hash_FindHashEntry(&uc->calls, (const char *) crn)) == NULL)
+    if ((entry = hash_FindHashEntry(&uc->calls, (const void *) (intptr_t) crn)) == NULL)
         return  NULL;
     /*endif*/
     return  (uc_call_t *) hash_GetHashValue(entry);
@@ -906,7 +1031,7 @@ uc_call_t *uc_getcall(uc_t *uc, int crn, int extra)
     uc_call_t *call;
     int new;
 
-    entry = hash_CreateHashEntry(&uc->calls, (const char *) crn, &new);
+    entry = hash_CreateHashEntry(&uc->calls, (const void *) (intptr_t) crn, &new);
     if (!new)
         return  (uc_call_t *) hash_GetHashValue(entry);
     /*endif*/
@@ -938,7 +1063,7 @@ uc_call_t *uc_createcall(uc_t *uc, int extra)
         /*endif*/
         crn = uc->next_crn | 0x8000;
     }
-    while (hash_FindHashEntry(&uc->calls, (const char *) crn));
+    while (hash_FindHashEntry(&uc->calls, (const void *) (intptr_t) crn));
     return  uc_getcall(uc, crn, extra);
 }
 /*- End of function --------------------------------------------------------*/
@@ -948,7 +1073,7 @@ void uc_destroycall(uc_t *uc, int crn)
     hash_HashEntry_t *entry;
 
     uc_log(uc, UC_LOG_FLOW, "Destroying call with CRN %d\n", crn);
-    if ((entry = hash_FindHashEntry(&uc->calls, (const char *) crn)) == NULL)
+    if ((entry = hash_FindHashEntry(&uc->calls, (const void *) (intptr_t) crn)) == NULL)
     {
         uc_error("Can't find CRN %d to destroy its call!\n", crn);
         return;
