@@ -24,7 +24,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: unicall.h,v 1.12 2005/01/28 23:29:47 steveu Exp $
+ * $Id: unicall.h,v 1.19 2005/07/10 15:31:09 steveu Exp $
  */
 
 /*! \file */
@@ -45,7 +45,6 @@
 #define UC_MODE_PEER                            0x03
 
 /* Call clearing causes, based on the ISDN names and values. */
-
 enum
 {
     /* Normal class */
@@ -287,7 +286,7 @@ enum
 
 /*! 
     UC_EVENT_ACCEPTED indicates completion of call acceptance. It is the
-    normal response to a uc_AcceptCall. Note that is af a call clears
+    normal response to a UC_OP_ACCEPTCALL. Note that is af a call clears
     during call acceptance, this event might never occur.
 */
     UC_EVENT_ACCEPTED,
@@ -342,14 +341,14 @@ enum
 
 /*! 
     UC_EVENT_FARDISCONNECTED indicates the far end has cleared the call first.
-    uc_DropCall should be used to clear the call from our end.
+    UC_OP_DROPCALL should be used to clear the call from our end.
 */
     UC_EVENT_FARDISCONNECTED,
 
 /*! 
-    UC_EVENT_DROPCALL follows a call to uc_DropCall. It indicates we cleared
+    UC_EVENT_DROPCALL follows a call to UC_OP_DROPCALL. It indicates we cleared
     forward first, and the far end has now responded by clearing backwards.
-    uc_ReleaseCall should be used to release resources used by the call.
+    UC_OP_RELEASECALL should be used to release resources used by the call.
 */
     UC_EVENT_DROPCALL,
 
@@ -375,13 +374,13 @@ enum
 
 /*!
     UC_EVENT_LOCALBLOCKED indicates the local end has completed its blocking operation
-    in response to a uc_Block() call.
+    in response to a UC_OP_BLOCK call.
 */
     UC_EVENT_LOCALBLOCKED,
 
 /*!
     UC_EVENT_LOCALUNBLOCKED indicates the local end has completed its unblocking operation
-    in response to a uc_Block() call.
+    in response to a UC_OP_BLOCK call.
 */
     UC_EVENT_LOCALUNBLOCKED,
 
@@ -398,6 +397,22 @@ enum
     UC_EVENT_USRINFO
 };
 
+enum
+{
+    UC_ECHO_CANCEL_OFF = 0,
+    UC_ECHO_CANCEL_ON,
+    UC_ECHO_CANCEL_NOTRAINING,
+    UC_ECHO_CANCEL_TRAINING
+};
+
+enum
+{
+    UC_SWITCHING_FREE = 0,
+    UC_SWITCHING_UNMUTE,
+    UC_SWITCHING_MUTE,
+    UC_SWITCHING_CONNECT
+};
+
 #define UC_MAXPHONENUMLEN           32
 #define UC_MAXPHONENAMELEN          256
 
@@ -728,6 +743,49 @@ extern int uc_get_call_state(uc_t *uc, uc_crn_t crn, int *state);
 */
 extern int uc_call_control(uc_t *uc, int op, uc_crn_t crn, void *data);
 
+/*! Open a channel
+    \return the file descriptor, or -1 if there is an error.
+*/
+int uc_channel_open(const char *protocol,
+                    const char *variant,
+                    int mode,
+                    char *dev);
+
+/*! Close a channel
+    \return 0 for OK, or -1 if there is an error.
+*/
+int uc_channel_close(const char *protocol,
+                     const char *variant,
+                     int mode,
+                     int fd);
+
+/*! Set the receive and transmit gains for a channel
+    \param uc The UniCall context
+    \param ch The channel number
+    \param rxgain
+    \param txgain
+    \return A UniCall completion code.
+*/
+int uc_channel_gains(uc_t *uc, int ch, float rxgain, float txgain);
+
+/*! Control echo cancellation
+    \param uc The UniCall context
+    \param ch The channel number
+    \param op Operation to perform
+    \return A UniCall completion code.
+*/
+int uc_channel_echo_cancel(uc_t *uc, int ch, int op);
+
+/*! Control circuit switching
+    \param uc The UniCall context
+    \param ch The channel number
+    \param op Operation to perform
+    \param dest Destination channel or group
+    \param parms Parameters
+    \return A UniCall completion code.
+*/
+int uc_channel_switching(uc_t *uc, int ch, int op, int dest, int parms);
+
 /*! Flush a channel's audio buffers.
     \param uc The UniCall context
     \param ch The channel number
@@ -739,6 +797,22 @@ extern int uc_channel_flush(uc_t *uc, int ch, int which);
 /*! \brief Get the device handles associated with a channel. */
 extern int uc_get_device_handles(uc_t *uc, int ch, int *fe, int *be);
 
+/*! \brief Get a channel's native codec.
+    \param uc The UniCall context
+    \param ch The channel number */
+extern int uc_channel_get_native_codec(uc_t *uc, int ch);
+
+/*! \brief Get the codec currently used at the API (read/write) interface.
+    \param uc The UniCall context
+    \param ch The channel number */
+extern int uc_channel_get_api_codec(uc_t *uc, int ch);
+
+/*! \brief Set the codec to be used at the API (read/write) interface.
+    \param uc The UniCall context
+    \param ch The channel number
+    \param codec The codec */
+extern int uc_channel_set_api_codec(uc_t *uc, int ch, int codec);
+
 /*! \brief Write to a channel. */
 extern int uc_channel_write(uc_t *uc, int ch, uint8_t *buf, int len);
 
@@ -847,17 +921,12 @@ extern int uc_set_channel_read_codec_callback(uc_t *uc, int ch, uc_channel_read_
 /*! \brief Specify a callback routine. */
 extern int uc_set_channel_write_codec_callback(uc_t *uc, int ch, uc_channel_write_codec_callback_t *func, void *user_data);
 
-/*! \brief Get a channel's native codec. */
-extern int uc_get_native_codec(uc_t *uc, int ch);
-/*! \brief Get the codec currently used at the API (read/write) interface. */
-extern int uc_get_api_codec(uc_t *uc, int ch);
-/*! \brief Set the codec to be used at the API (read/write) interface. */
-extern int uc_set_api_codec(uc_t *uc, int ch, int codec);
-
 /*! \brief Specify the logging level for a channel, and a tag to be used in the
            log to identify the channel. */
 extern void uc_set_logging(uc_t *uc, int level, int format, const char *tag);
 
+extern char *uc_error_message(uc_t *uc, int code);
+
 extern void uc_set_message_handler(void (*func)(char *text));
 extern void uc_set_error_handler(void (*func)(char *text));
 extern int uc_str2protocol_variant(int protocol_class, char *variant);
@@ -1140,7 +1209,7 @@ enum
     
     /*! Accept a call that has been offered in a UC_EVENT_OFFERED (or a subsequent
          UC_EVENT_MOREDIGITS) event.
-    
+   
         Accept a call that has been offered in a UC_EVENT_OFFERED event. This means
         the call is to a valid destination, which is free to accept calls. The call
         is then effectively ringing.
@@ -1149,7 +1218,7 @@ enum
     
     /*! Answer a call that has been offered in a UC_EVENT_OFFERED (or a subsequent
         UC_EVENT_MOREDIGITS) event.
-
+   
         Answer a call that has been offered in a UC_EVENT_OFFERED event. The call
         may have already been accepted, using uc_AcceptCall, or this step may have
         been skipped.  This should result in either a UC_EVENT_ANSWERED event, when
@@ -1166,7 +1235,7 @@ enum
     UC_OP_DROPCALL,
     
     /*! Release a call, and free any resources associated with it.
-
+   
         Release a call, and free any resources associated with it. Once this
         operation is performed, the CRN will no longer be valid. This should
         result in a UC_EVENT_RELEASECALL event, when any slow activities related
